#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import sys

from pyglet import clock
from pyglet.gl import *
from pyglet.window import Window
from pyglet.window import key

class GameState(object):
    has_exit = False
    queued_state = None

    def activate(self, window):
        pass

    def deactivate(self, window):
        pass

    def draw(self):
        pass

    def update(self, dt):
        pass

class GameWindow(Window):
    def __init__(self, title, **kwargs):
        super(GameWindow, self).__init__(**kwargs)
        self.set_caption(title)
        self.states = []

    def push_state(self, state):
        if self.states:
            self.states[-1].deactivate(self)
            self.states[-1].queued_state = None

        self.states.append(state)
        self.push_handlers(state)
        state.activate(self)

    def pop_state(self):
        self.states[-1].deactivate(self)
        self.states.pop()
        self.pop_handlers()

        if self.states:
            self.states[-1].activate(self)
        else:
            self.exit_handler.has_exit = True

    def run(self):
        while not self.has_exit:
            while self.states and self.states[-1].has_exit:
                self.pop_state()

            dt = clock.tick()
            # Filter out insane spikes (e.g. due to another process, or
            # loading)
            if dt > 0.1:
                dt = 0.01

            self.dispatch_events()
            if self.states:
                self.states[-1].update(dt)
                self.states[-1].draw()
            self.flip()

            if self.states and self.states[-1].queued_state:
                self.push_state(self.states[-1].queued_state)
