#!/usr/bin/env python

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: $'

import sys

from pyglet.gl import *
from pyglet import clock
from pyglet import font

import game

class Menu(game.GameState):
    options = []
    text_color = (0, 0, 0, 1)

    def __init__(self):
        self.selected_option = None
        self.selected_hot = False

        self.label_font = font.load('Arial', 24., bold=True)
        self.labels = []

        width = 0
        height = 0
        for name in self.options[::-1]:
            if name is None:
                height += 30
                continue
            label = font.Label(self.label_font, name, color=self.text_color)
            label.y = height + 5
            width = max(width, label.width)
            height += label.height
            self.labels.append(label)
            height += 10

        self.width = width
        self.height = height
        self.x = 0
        self.y = 0

        self.has_exit = False

    def activate(self, window):
        self._width = window.width
        self._height = window.height
        self.x = (window.width - self.width) / 2
        self.y = (window.height - self.height) / 2

    def update(self, dt):
        pass
        
    def draw(self):
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        glOrtho(0, self._width, 0, self._height, -1, 1)

        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

        glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT)
        glDisable(GL_LIGHTING)
        glDisable(GL_DEPTH_TEST)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        self.draw_background()
        
        glTranslatef(self.x, self.y, 0)
        for label in self.labels:
            label.draw()

        glPopAttrib()
        
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()

        glMatrixMode(GL_MODELVIEW)

    def draw_background(self):
        pass
        
    def on_key_press(self, symbol, modifiers):
        self.has_exit = True
        return True

    def on_key_release(self, symbol, modifiers):
        return True

    def get_selected(self, x, y):
        if x > self.x and x < self.x + self.width:
            y -= self.y
            for label in self.labels:
                if y > label.y + label.font.descent and \
                   y < label.y + label.height + label.font.descent:
                    return label
        return None

    def on_mouse_motion(self, x, y, dx, dy):
        self.selected_option = self.get_selected(x, y)
        return True

    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        return self.on_mouse_motion(x, y, dx, dy)

    def on_mouse_press(self, button, x, y, modifiers):
        if self.selected_option:
            self.selected_hot = True
        return True

    def on_mouse_release(self, button, x, y, modifiers):
        selected = self.get_selected(x, y)
        if selected is not None:
            self.on_select(selected)
        self.selected_hot = False
        return True
    
class PopupMenu(Menu):
    def __init__(self, fps):
        super(PopupMenu, self).__init__()
        self.fps = fps

    def draw_background(self):
        glPushAttrib(GL_CURRENT_BIT)

        def rect(x, y, width, height):
            glBegin(GL_QUADS)
            glVertex2f(x, y)
            glVertex2f(x + width, y)
            glVertex2f(x + width, y + height)
            glVertex2f(x, y + height)
            glEnd()

        glColor4f(0, 0, 0, 0.5)
        rect(0, 0, self._width, self._height)

        pad = 10
        y = self.y + self.label_font.descent - pad
        x = self.x - pad

        glColor4f(1, 1, 1, 0.7)
        rect(x, y, self.width + pad * 2, self.height + pad * 2)

        if self.selected_option:
            if self.selected_hot:
                glColor4f(1., 1., 1., .8)
            else:
                glColor4f(.8, .8, 1., .8)
            label = self.selected_option
            rect(self.x + label.x - pad, 
                 self.y + label.y + label.font.descent - 5,
                 self.width + pad * 2, 
                 label.height + 10)

        glPopAttrib()

    def draw(self):
        self.fps.draw()

        super(PopupMenu, self).draw()

class GameMenu(PopupMenu):
    options = [
        'Resume Game',
        'Retry Stage',
        'Main Menu',
        'Exit',
    ]


    def on_select(self, label):
        if label.text == 'Resume Game':
            pass
        elif label.text == 'Retry Stage':
            self.fps.level.retry_stage()
        elif label.text == 'Main Menu':
            self.fps.has_exit = True
        elif label.text == 'Exit':
            sys.exit()
        
        self.has_exit = True

class CheatMenu(PopupMenu):
    options = [
        'Resume Game',
        'Advance Stage',
        'Unlimited Weapon Charge',
        'Unlimited Health',
        'FPS Counter'
    ]


    def on_select(self, label):
        if label.text == 'Resume Game':
            pass
        elif label.text == 'Advance Stage':
            self.fps.level.advance_stage()
        elif label.text == 'Unlimited Weapon Charge':
            self.fps.level.weapon_charge = 10000000.
            self.fps.level.update_weapon_materials()
        elif label.text == 'Unlimited Health':
            self.fps.level.set_health(1000000.)
        elif label.text == 'FPS Counter':
            self.fps.clock_display = clock.ClockDisplay()
        
        self.has_exit = True


